@@ -2,17 +2,16 @@ from __future__ import annotations
 from typing import Callable
 
 from numpy.typing import ArrayLike
-import tqdm
+import xarray as xr
 
-from magnify.assay import Assay
 import magnify.registry as registry
 import magnify.utils as utils
 
 
 class Pipeline:
     def __init__(self, reader: str):
-        self.reader: Callable[[ArrayLike | str], Assay] = registry.readers.get(reader)()
-        self.components: list[Callable[[Assay], Assay]] = []
+        self.reader: Callable[[ArrayLike | str], xr.Dataset] = registry.readers.get(reader)()
+        self.components: list[Callable[[xr.Dataset], xr.Dataset]] = []
 
     def __call__(
         self,
@@ -21,14 +20,13 @@ class Pipeline:
         search_on: str = "egfp",
         times: Sequence[int] | None = None,
         channels: Sequence[str] | None = None,
-        progress_bar: bool = False,
         **kwargs,
-    ) -> Assay:
+    ) -> xr.Dataset | list[xr.Dataset]:
         inputs = self.reader(
             data=data, names=names, search_on=search_on, times=times, channels=channels
         )
         assays = []
-        for assay in tqdm.tqdm(inputs, disable=not progress_bar):
+        for assay in inputs:
             for name, component in self.components:
                 assay = component(assay, **utils.valid_kwargs(kwargs, component))
 
